'''
Copyright (c) 2019-2021 TIV Consultancy. All right reserved

No redistribution in any form allowed.
'''

from __future__ import absolute_import
from mxnet import nd
from mxnet.gluon.loss import Loss

class LogSoftmaxLossWeighted(Loss):
    r"""LogSoftmaxLossWeighted

    Parameters
    ----------
    weight : float or None
        Global scalar weight for loss.
    batch_axis : int, default 0
        The axis that represents mini-batch.
    """
    def __init__(self, weight=0.1, batch_axis=0, **kwargs):
        super(LogSoftmaxLossWeighted, self).__init__(None, batch_axis, **kwargs)
        self._minDiv = 1E-3
        self._axis = 1
        self.weight = weight

    def hybrid_forward(self, F, output, label):
        """Compute loss"""
        #label = nd.expand_dims(label, axis=1)
        #pred = nd.log_softmax(output, axis=self._axis)
        loss = -nd.pick(nd.log_softmax(output, axis=self._axis), label, axis=self._axis)
        weighting = nd.where(label == 0, nd.ones_like(label) * self.weight,label)
        #plot_mx_array(nd.array(weighting[0]))
        loss = (loss * weighting)
        integral = loss.sum(axis=0, exclude=True)
        weight_sum = weighting.sum(axis=0, exclude=True)
        return integral/weight_sum


class LogSoftmaxLossWeighted_localWeight(Loss):
    r"""LogSoftmaxLossWeighted_localWeight

    Parameters
    ----------
    weight : float or None
        Global scalar weight for loss.
    batch_axis : int, default 0
        The axis that represents mini-batch.
    """
    def __init__(self, weight=0.1, batch_axis=0, **kwargs):
        super(LogSoftmaxLossWeighted_localWeight, self).__init__(None, batch_axis, **kwargs)
        self._minDiv = 1E-3
        self._axis = 1
        self.weight = weight

    def hybrid_forward(self, F, output, label, weight):
        """Compute loss"""
        #label = nd.expand_dims(label, axis=1)
        #pred = nd.log_softmax(output, axis=self._axis)
        loss = -nd.pick(nd.log_softmax(output, axis=self._axis), label, axis=self._axis)
        weighting = nd.where(weight == 0, nd.ones_like(label) * self.weight,weight)
        #plot_mx_array(nd.array(label[0]))
        #plot_mx_array(nd.array(weighting[0]))
        loss = (loss * weighting)
        integral = loss.sum(axis=0, exclude=True)
        weight_sum = weighting.sum(axis=0, exclude=True)
        return integral/weight_sum