'''
Copyright (c) 2019-2021 TIV Consultancy. All right reserved

No redistribution in any form allowed.
'''

from mxnet import npx


class ParamFCNTIV:
    r"""ParamFCNTIV

    Class to store and vary parameters
    Parameters
    ----------
    weight : float or None
        Global scalar weight for loss.
    batch_axis : int, default 0
        The axis that represents mini-batch.
    """
    def __init__(self, batch_size=20, num_classes=2, ctx=npx.gpu(), num_epoch=250, weight_decay=0.001,
                 momentum_train=0.9, num_workers=4, ident="tiv_FCN_stdset", learn_rate=0.001):
        self.batch_size = batch_size
        self.num_classes = num_classes  # Foreground/Background Classification
        self.ctx = ctx
        self.num_epoch = num_epoch
        self.weight_decay = weight_decay
        self.momentum_train = momentum_train
        self.num_workers = num_workers
        self.ident = ident
        self.learn_rate = learn_rate

    def create_variation_batch_size(self, start_batch=5, end_batch=35, step_batch=10, ident_base="fcn_tiv_batchvar_"):
        #Creates a list with variation in batch size
        list_settings_batch = []
        for x in range(start_batch, end_batch+step_batch, step_batch):
            list_settings_batch.append(
                ParamFcnTIV(batch_size=x, ident=ident_base + str(x), num_classes=self.num_classes, ctx=self.ctx,
                              num_epoch=self.num_epoch, weight_decay=self.weight_decay,
                              momentum_train=self.momentum_train, num_workers=self.num_workers))
        return list_settings_batch

    def create_variation_wd_size(self, start_wd=0.0001, end_wd=1, mult_wd=10, ident_base="fcn_tiv_wdvar_"):
        # Creates a list with variation for weight decay
        import math
        count = math.log(end_wd / start_wd, mult_wd)
        list_settings_batch = []
        wd = start_wd
        for x in range(int(count+1)):
            list_settings_batch.append(
                ParamFcnTIV(weight_decay=wd, ident=ident_base + str(x), batch_size=self.batch_size,
                              num_classes=self.num_classes, ctx=self.ctx, num_epoch=self.num_epoch,
                              momentum_train=self.momentum_train, num_workers=self.num_workers))
            wd = wd * mult_wd
        return list_settings_batch

    def create_variation_mom_size(self, start_mom=0.1, end_mom=1, mult_mom=1.58, ident_base="fcn_tiv_wdvar_"):
        # Creates a list with variation for the momentum
        import math
        count = math.log(end_mom / start_mom, mult_mom)
        list_settings_batch = []
        mom = start_mom
        for x in range(int(count+1)):
            list_settings_batch.append(
                ParamFcnTIV(momentum_train=mom, ident=ident_base + str(x), weight_decay=self.weight_decay,
                              batch_size=self.batch_size, num_classes=self.num_classes, ctx=self.ctx,
                              num_epoch=self.num_epoch, num_workers=self.num_workers))
            mom = mom * mult_mom
        return list_settings_batch
